@@ -1,7 +1,10 @@
 import os
 import glob
+import json
 from logging import getLogger
 
+import xmltodict
+
 from .exceptions import FastLabelInvalidException
 from .api import Api
 from fastlabel import converters, utils
@@ -246,7 +249,7 @@ class Client:
         self, project: str,
         offset: int = None,
         limit: int = 1000,
-    ) -> list:
+    ) -> dict:
         """
         Returns a map of task ids and names.
         e.g.) {
@@ -390,6 +393,7 @@ class Client:
         name: str,
         file_path: str,
         status: str = None,
+        annotations: list = [],
         tags: list = [],
     ) -> str:
         """
@@ -399,6 +403,7 @@ class Client:
         name is an unique identifier of task in your project. (Required)
         file_path is a path to data. Supported extensions are png, jpg, jpeg. (Required)
         status can be 'registered', 'in_progress', 'completed', 'skipped', 'in_review', 'send_backed', 'approved', 'customer_in_review', 'customer_send_backed', 'customer_approved'. (Optional)
+        annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
         endpoint = "tasks/video"
@@ -409,6 +414,10 @@ class Client:
         payload = {"project": project, "name": name, "file": file}
         if status:
             payload["status"] = status
+        if annotations:
+            for annotation in annotations:
+                annotation["content"] = name
+            payload["annotations"] = annotations
         if tags:
             payload["tags"] = tags
         return self.api.post_request(endpoint, payload=payload)
@@ -447,12 +456,58 @@ class Client:
 
     # Task Convert
 
-    def to_coco(self, tasks: list) -> dict:
-        """
-        Convert tasks to COCO format.
+    def export_coco(self, tasks: list, output_dir: str = os.path.join("output", "coco")) -> None:
         """
+        Convert tasks to COCO format and export as a file.
 
-        return converters.to_coco(tasks)
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/coco). (Optional)
+        """
+        coco = converters.to_coco(tasks)
+        os.makedirs(output_dir, exist_ok=True)
+        file_path = os.path.join(output_dir, "annotations.json")
+        with open(file_path, 'w') as f:
+            json.dump(coco, f, indent=4, ensure_ascii=False)
+
+    def export_yolo(self, tasks: list, output_dir: str = os.path.join("output", "yolo")) -> None:
+        """
+        Convert tasks to YOLO format and export as files.
+
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/yolo). (Optional)
+        """
+        annos, categories = converters.to_yolo(tasks)
+        for anno in annos:
+            file_name = anno["filename"]
+            basename = utils.get_basename(file_name)
+            file_path = os.path.join(
+                output_dir, "annotations", basename + ".txt")
+            os.makedirs(os.path.dirname(file_path), exist_ok=True)
+            with open(file_path, 'w', encoding="utf8") as f:
+                for obj in anno["object"]:
+                    f.write(obj)
+                    f.write("\n")
+        with open(os.path.join(output_dir, "classes.txt"), 'w', encoding="utf8") as f:
+            for category in categories:
+                f.write(category["name"])
+                f.write("\n")
+
+    def export_pascalvoc(self, tasks: list, output_dir: str = os.path.join("output", "pascalvoc")) -> None:
+        """
+        Convert tasks to Pascal VOC format as files.
+
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/pascalvoc). (Optional)
+        """
+        pascalvoc = converters.to_pascalvoc(tasks)
+        for voc in pascalvoc:
+            file_name = voc["annotation"]["filename"]
+            basename = utils.get_basename(file_name)
+            file_path = os.path.join(output_dir, basename + ".xml")
+            os.makedirs(os.path.dirname(file_path), exist_ok=True)
+            xml = xmltodict.unparse(voc, pretty=True, full_document=False)
+            with open(file_path, 'w', encoding="utf8") as f:
+                f.write(xml)
 
     # Annotation
 
@@ -507,7 +562,7 @@ class Client:
         type: str,
         value: str,
         title: str,
-        color: str,
+        color: str = None,
         attributes: list = []
     ) -> str:
         """
@@ -517,7 +572,7 @@ class Client:
         type can be 'bbox', 'polygon', 'keypoint', 'classification', 'line', 'segmentation'. (Required)
         value is an unique identifier of annotation in your project. (Required)
         title is a display name of value. (Required)
-        color is hex color code like #ffffff. (Required)
+        color is hex color code like #ffffff. (Optional)
         attributes is a list of attribute. (Optional)
         """
         endpoint = "annotations"
@@ -526,8 +581,9 @@ class Client:
             "type": type,
             "value": value,
             "title": title,
-            "color": color
         }
+        if color:
+            payload["color"] = color
         if attributes:
             payload["attributes"] = attributes
         return self.api.post_request(endpoint, payload=payload)
@@ -597,3 +653,142 @@ class Client:
         """
         endpoint = "annotations/" + annotation_id
         self.api.delete_request(endpoint)
+
+    # Project
+
+    def find_project(self, project_id: str) -> dict:
+        """
+        Find a project.
+        """
+        endpoint = "projects/" + project_id
+        return self.api.get_request(endpoint)
+
+    def find_project_by_slug(self, slug: str) -> dict:
+        """
+        Find a project by slug.
+
+        slug is slug of your project. (Required)
+        """
+        projects = self.get_projects(slug=slug)
+        if not projects:
+            return None
+        return projects[0]
+
+    def get_projects(
+        self,
+        slug: str = None,
+        offset: int = None,
+        limit: int = 100,
+    ) -> list:
+        """
+        Returns a list of projects.
+        Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
+
+        slug is slug of your project. (Optional)
+        offset is the starting position number to fetch. (Optional)
+        limit is the max number to fetch. (Optional)
+        """
+        if limit > 1000:
+            raise FastLabelInvalidException(
+                "Limit must be less than or equal to 1000.", 422)
+        endpoint = "projects"
+        params = {}
+        if slug:
+            params["slug"] = slug
+        if offset:
+            params["offset"] = offset
+        if limit:
+            params["limit"] = limit
+        return self.api.get_request(endpoint, params=params)
+
+    def get_project_id_slug_map(
+        self,
+        offset: int = None,
+        limit: int = 1000,
+    ) -> dict:
+        """
+        Returns a map of project ids and slugs.
+        e.g.) {
+                "88e74507-07b5-4607-a130-cb6316ca872c", "image-bbox-slug",
+                "fe2c24a4-8270-46eb-9c78-bb7281c8bdgs", "image-video-slug"
+              }
+        Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
+
+        offset is the starting position number to fetch. (Optional)
+        limit is the max number to fetch. (Optional)
+        """
+        if limit > 1000:
+            raise FastLabelInvalidException(
+                "Limit must be less than or equal to 1000.", 422)
+        endpoint = "projects/map/id-slug"
+        params = {}
+        if offset:
+            params["offset"] = offset
+        if limit:
+            params["limit"] = limit
+        return self.api.get_request(endpoint, params=params)
+
+    def create_project(
+        self,
+        type: str,
+        name: str,
+        slug: str,
+        is_bitmap: bool = False,
+        job_size: int = 10,
+        use_annotation_service: bool = False
+    ) -> str:
+        """
+        Create a project.
+
+        type can be 'image_bbox', 'image_polygon', 'image_keypoint', 'image_line', 'image_segmentation', 'image_classification', 'image_all', 'multi_image_bbox', 'multi_image_polygon', 'multi_image_keypoint', 'multi_image_line', 'multi_image_segmentation', 'video_bbox', 'video_single_classification'. (Required)
+        name is name of your project. (Required)
+        slug is slug of your project. (Required)
+        is_bitmap is whether to be annotated by pixel. (Optional)
+        job_size is the number of tasks the annotator gets at one time. (Optional)
+        use_annotation_service is whether to request FastLabel to provide annotation service or not. (Optional)
+        """
+        endpoint = "projects"
+        payload = {
+            "type": type,
+            "name": name,
+            "slug": slug,
+        }
+        if is_bitmap:
+            payload["isBitmap"] = is_bitmap
+        if job_size:
+            payload["jobSize"] = job_size
+        if use_annotation_service:
+            payload["useAnnotationService"] = use_annotation_service
+        return self.api.post_request(endpoint, payload=payload)
+
+    def update_project(
+        self,
+        project_id: str,
+        name: str = None,
+        slug: str = None,
+        job_size: int = None,
+    ) -> str:
+        """
+        Update a project.
+
+        project_id is an id of the project. (Required)
+        name is name of your project. (Optional)
+        slug is slug of your project. (Optional)
+        job_size is the number of tasks the annotator gets at one time. (Optional)
+        """
+        endpoint = "projects/" + project_id
+        payload = {}
+        if name:
+            payload["name"] = name
+        if slug:
+            payload["slug"] = slug
+        if job_size:
+            payload["jobSize"] = job_size
+        return self.api.put_request(endpoint, payload=payload)
+
+    def delete_project(self, project_id: str) -> None:
+        """
+        Delete a project.
+        """
+        endpoint = "projects/" + project_id
+        self.api.delete_request(endpoint)
